/*
 * mvclock.c
 *
 * Moving clock blanker
 *
 * Author:  Shuro Shindo <shu@pegasus.rim.or.jp>
 * Created: Apr. 6,1996
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/time.h>
#include "vga.h"
#include "vgagl.h"


#define	BlockWidth	( 8 * 8 )
#define	BlockHeight	8
#define	DeltaX		1
#define DeltaY 		1
#define FontColor       14	/* yellow */
#define BackgroundColor 8	/* dark gray */

/* If moving speed is too slow, decrease Delay */
#define Delay		( 20 * 1000 )


/* Following macro variables are defined in vgagl.h.
   WIDTH, HEIGHT, COLORS, BYTESPERPIXEL
*/

typedef struct point_s {
	int	x;
	int	y;
}	point_t;

void handler( int signal )
{
	vga_setmode( TEXT );
	exit( -1 );
}

int initscreen( void )
{
	int	ret;
	int	mode;

	ret = vga_init();
	if ( ret != 0 ) {
		fprintf( stderr, "Can't initialize vga." );
		return( 1 );
	}
	ret = vga_setmode( G640x480x256 );
	if ( ret != 0 ) {
		ret = vga_setmode( G640x480x16 );
		if ( ret != 0 ) {
			fprintf( stderr, "Can't set graphics mode to 640x480x[16|256]." );
			return( 1 );
		}
	}
	mode = vga_getcurrentmode();
	gl_setcontextvga( mode );
	gl_clearscreen( BackgroundColor );
	return( 0 );
}

void initfont( int color )
{
	void *font;

        font = malloc( 256 * 8 * 8 * BYTESPERPIXEL );
        gl_expandfont( 8, 8, color, gl_font8x8, font );
        gl_setfont( 8, 8, font );
        gl_setwritemode( WRITEMODE_MASKED );
}
	
void draw( point_t *p )
{
	char s[ 9 ];		/* HH:MM:SS */
	time_t t;

	time( &t );
	strftime( s, sizeof( s ), "%H:%M:%S", localtime( &t ));
	gl_write( p->x, p->y, s );
#if 0
	printf( "(%d,%d):%s\n", p->x, p->y, s );
#endif
}

void calcpos( point_t *p, int w, int h, int *dx, int *dy )
{
	if ( *dx >= 0 ) {
		if ( p->x + w >= WIDTH ) {
			*dx = - DeltaX;
		}
	}
	else {
		if ( p->x <= 0 ) {
			*dx = DeltaX;
		}
	}
	if ( *dy >= 0 ) {
		if ( p->y + h >= HEIGHT ) {
			*dy = - DeltaY;
		}
	}
	else {
		if ( p->y <= 0 ) {
			*dy = DeltaY;
		}
	}
}

void main( void )
{
	point_t	p1 = { 0, 0 };
	int	width = BlockWidth;
	int	height = BlockHeight;
	int	dx = DeltaX;
	int	dy = DeltaY;

	int	ret;
	char	*buf;
	int	color = FontColor;


	/* initialization */
	signal( SIGQUIT, handler );

	ret = initscreen();
	if ( ret != 0 ) {
		exit( 1 );
	}
	printf( "width:%d, height:%d, colors:%d, bytes/pixel:%d\n", 
		WIDTH, HEIGHT, COLORS, BYTESPERPIXEL );

	buf = malloc( width * height * BYTESPERPIXEL );
	if ( buf == NULL ) {	/* not enough memory */
		exit( 2 );
	}

	initfont( color );

	for ( ; ; ) {

		/* save */
		gl_getbox( p1.x, p1.y, width, height, buf );

		/* draw */
		draw( &p1 );

		/* calc new position */
		calcpos( &p1, width, height, &dx, &dy );

		/* timing adjustment */
		usleep( Delay );

		/* restore */
		gl_putbox( p1.x, p1.y, width, height, buf );

		/* set new position */
		p1.x += dx;
		p1.y += dy;
	}
}
